#include <assert.h>
#include <stdlib.h>
#include "List.h"

/// Returns TRUE if the list is empty.
BOOL listIsEmpty(List *list)
{
	assert(list != NULL);
	if (list->size == 0)
		return TRUE;
	else
		return FALSE;
}

/// Initialize the list.
void initList(List *list)
{
	assert(list != NULL);

	list->size = 0;
	list->firstNode = NULL;
}

/// Print the contents of the list.
void printList(List *list)
{
	ListNode *node = NULL;
	assert(list != NULL);

	if (listIsEmpty(list) == TRUE)
		printf("List is empty!\n");

	for (node = list->firstNode; node != NULL; node = node->next) {
		printf(TYPE_MODIFIER" ", node->data);
	}
}

/// Insert data into the list
void insertData(List *list, Type data)
{
	ListNode *newNode = NULL;
	assert(list != NULL);

	if (exists(list, data) == TRUE)
		return;
	
	newNode = (ListNode *)malloc(sizeof(ListNode));

	// Check for memory allocation errors
	if (newNode == NULL) {
		fprintf(stderr, "Not Enough memory!\nDeleting the list and Exiting!!\n");
		clearList(list);
		exit(1);
	}
	newNode->data = data;
	newNode->next = NULL;

	// If the list is empty, create the first node
	if (listIsEmpty(list) == TRUE) {
		list->firstNode = newNode;
	}
	else {
		// We have a new first node
		newNode->next = list->firstNode;
		list->firstNode = newNode;
	}
	list->size++;
}

/// Delete the first occurence of data from the list. If deletion was succesfull, return TRUE, else FALSE.
BOOL deleteData(List *list, Type data)
{
	ListNode *cur = NULL;
	ListNode *prev = NULL;

	assert(list != NULL);

	cur = list->firstNode;
	while ((cur != NULL) && (data != cur->data)) {
		prev = cur;
		cur = cur->next;
	}
	if ((cur != NULL) && (cur->data == data)) {
		// Found a node, delete it.
//		ListNode *tmp = node;
		if (cur == list->firstNode) {
			list->firstNode = list->firstNode->next;
			free(cur);
		}
		else {
			prev->next = cur->next;
			free(cur);
		}
		list->size--;
		return TRUE;
	}
	else {
		fprintf(stderr, "Warning: Value "TYPE_MODIFIER" was not found in the list!\n", data);
		return FALSE;
	}
}

/// Access the data at position index. Return NULL if unsuccesfull.
ListNode *accessNode(List *list, int index)
{
	ListNode *node = NULL;

	assert(list != NULL);

	if (index >= list->size)
		return NULL;
	else {
		int i = 0;
		node = list->firstNode;
		for (i = 0; i < index; i++) {
			node = node->next;
		}
		return node;
	}
}

/// Clear the contents of the list.
void clearList(List *list)
{
	ListNode *next = NULL;
	ListNode *cur = NULL;

	assert(list != NULL);

	if (listIsEmpty(list) == FALSE) {
		cur = list->firstNode;
		next = cur->next;
		while (next != NULL) {
			next = cur->next;
			free(cur);
			cur = next;
		}
	}	
}

/// Find the first node with key value key.
ListNode *findNode(List *list, Type key)
{
	ListNode *cur = NULL;

	assert(list != NULL);

	cur = list->firstNode;
	while ((cur != NULL) && (key != cur->data)) {
		cur = cur->next;
	}
	if ((cur != NULL) && (cur->data == key)) {
		// Found a node, return it.
		return cur;
	}
	else {
		return NULL;
	}
}

/// Check if a value is in the list, NULL if non existant.
BOOL exists(List *list, Type key)
{
	if (findNode(list, key) != NULL)
		return TRUE;
	else
		return FALSE;
}
