#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "defns.h"
#include "ListGraph.h"
#include "Queue.h"


// create and initialize a new graph
ListGraph *lgCreate(GRAPHTYPE type, int numOfVertices)
{
	ListGraph *graph = (ListGraph *)malloc(sizeof(ListGraph));
	int i = 0;

	if (graph == NULL)
		ERROR("Memory Allocation Error!!!");

	graph->numOfVertices = numOfVertices;
	graph->type = type;

	// Init the adjacency lists
	graph->arrayOfLists = (List *)malloc(numOfVertices * sizeof(List));
	if (graph->arrayOfLists == NULL)
		ERROR("Memory Allocation Error!!!");

	for (i = 0; i < numOfVertices; i++)	{
		initList(&graph->arrayOfLists[i]);
	}

	// Init the visited array
	graph->visited = (int *)malloc(numOfVertices * sizeof(BOOL));
	if (graph->visited == NULL)
		ERROR("Memory Allocation Error!!!");

	for (i = 0; i < numOfVertices; i++) {
		graph->visited[i] = FALSE;
	}

	// Init the values array
	graph->values = (char *)malloc(numOfVertices * sizeof(char));
	for (i = 0; i < numOfVertices; i++) {
		graph->values[i] = 'a' + i;
	}

	return graph;
}

// Delete the graph
void lgDelete(ListGraph *graph)
{
	int i = 0;
	assert(graph != NULL);

	for (i = 0; i < graph->numOfVertices; i++)	{
		clearList(&graph->arrayOfLists[i]);
	}
	if (graph->arrayOfLists != NULL)
		free(graph->arrayOfLists);
	if (graph->visited != NULL)
		free(graph->visited);

	if (graph->values != NULL)
		free(graph->values);

	free(graph);
}

// Set an edge between vertices v1 and v2
void lgAddEdge(ListGraph *graph, int v1, int v2, int weight)
{
#ifdef _DEBUG
	if (weight == INVALID_WEIGHT)
		WARNING("Invalid weight inserted!!!");
#endif

	insertData(&graph->arrayOfLists[v1], v2);
	if (graph->type == UNDIRECTED)
		insertData(&graph->arrayOfLists[v2], v1);
}

// remove the edge between vertices v1 and v2
void lgRemoveEdge(ListGraph *graph, int v1, int v2)
{
	deleteData(&graph->arrayOfLists[v1], v2);
	if (graph->type == UNDIRECTED)
		deleteData(&graph->arrayOfLists[v2], v1);
}

// print the adjacency list
void lgPrint(ListGraph *graph)
{
	int i = 0;
	ListNode *node = NULL;

	fprintf(stdout, "The Adjacency List:\n");
	fprintf(stdout, "-------------------\n");
	for (i = 0; i < graph->numOfVertices; i++) {
		fprintf(stdout, "\t%c: ", graph->values[i]);
		for (node = graph->arrayOfLists[i].firstNode; node != NULL; node = node->next) {
			fprintf(stdout, "%c ", graph->values[node->data]);
		}
		fprintf(stdout, "\n");
	}

	fprintf(stdout, "The Edges:\n");
	fprintf(stdout, "----------\n");
	for (i = 0; i < graph->numOfVertices; i++) {
		fprintf(stdout, "\t");
		for (node = graph->arrayOfLists[i].firstNode; node != NULL; node = node->next) {
			fprintf(stdout, "%c-%c ", graph->values[i], graph->values[node->data]);
		}
		fprintf(stdout, "\n");
	}
}

// Visit all the nodes of the graph starting from v, in Depth First Order
void lgDepthFirstSearch(ListGraph *graph, int v)
{
	/////////////////////////////////////////
	// INSERT CODE HERE
	/////////////////////////////////////////
}

// Recursive DFS algorithm
void lgDFS(ListGraph *graph, int v)
{
	/////////////////////////////////////////
	// INSERT CODE HERE
	/////////////////////////////////////////
}

// Visit all the nodes of the graph starting from v, in Breadth First Order
void lgBreadthFirstSearch(ListGraph *graph, int v)
{
	/////////////////////////////////////////
	// INSERT CODE HERE
	/////////////////////////////////////////
}