#ifndef __LIST_GRAPH_H__
#define __LIST_GRAPH_H__

#include "defns.h"

#include "List.h"

/**
 * Represent a Graph as an Adjacent List.
 */
typedef struct _ListGraph {
	GRAPHTYPE type;			// Unidirected or directed?
	List *arrayOfLists;		// The array of adjacent lists
	int numOfVertices;		// The number of graph vertices (nodes)
	BOOL *visited;			// Indicated if  node was visited during a search or not
	char *values;			// The values for each node
} ListGraph;

// create and initialize a new graph
ListGraph *lgCreate(GRAPHTYPE type, int numOfVertices);

// Delete the graph
void lgDelete(ListGraph *graph);

// Set an edge with specified weight between vertices v1 and v2
// If the graph is directed, then the edge is from v1 to v2
// If the graph is undirected, the edge is unidirectional
void lgAddEdge(ListGraph *graph, int v1, int v2, int weight);

// Delete the edge between vertices v1 and v2
// If the graph is directed, then the edge is from v1 to v2
// If the graph is undirected, the edge is unidirectional
void lgRemoveEdge(ListGraph *graph, int v1, int v2);

// print the list graph
void lgPrint(ListGraph *graph);

// Visit all the nodes of the graph starting from v, in Depth First Order
void lgDepthFirstSearch(ListGraph *graph, int v);
void lgDFS(ListGraph *graph, int v);

// Visit all the nodes of the graph starting from v, in Breadth First Order
void lgBreadthFirstSearch(ListGraph *graph, int v);

#endif // __LIST_GRAPH_H__