#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "defns.h"
#include "MatrixGraph.h"
#include "Queue.h"


// create and initialize a new graph
MatrixGraph *mgCreate(GRAPHTYPE type, int numOfVertices)
{
	MatrixGraph *graph = (MatrixGraph *)malloc(sizeof(MatrixGraph));
	int i = 0;

	if (graph == NULL)
		ERROR("Memory Allocation Error!!!");

	graph->numOfVertices = numOfVertices;
	graph->type = type;

	// Init the adjacency matrix
	graph->matrix = (int **)malloc(numOfVertices * sizeof(int *));
	if (graph->matrix == NULL)
		ERROR("Memory Allocation Error!!!");

	for (i = 0; i < numOfVertices; i++)	{
		int j = 0;
		graph->matrix[i] = (int *)malloc(numOfVertices * sizeof(int));
		if (graph->matrix[i] == NULL)
			ERROR("Memory Allocation Error!!!");
		for (j = 0; j < numOfVertices; j++) {
			graph->matrix[i][j] = INVALID_WEIGHT;
		}
	}

	// Init the visited array
	graph->visited = (int *)malloc(numOfVertices * sizeof(BOOL));
	if (graph->visited == NULL)
		ERROR("Memory Allocation Error!!!");

	for (i = 0; i < numOfVertices; i++) {
		graph->visited[i] = FALSE;
	}

	// Init the values array
	graph->values = (char *)malloc(numOfVertices * sizeof(char));
	for (i = 0; i < numOfVertices; i++) {
		graph->values[i] = 'a' + i;
	}

	return graph;
}

// Delete the graph
void mgDelete(MatrixGraph *graph)
{
	int i = 0;
	assert(graph != NULL);

	for (i = 0; i < graph->numOfVertices; i++)	{
		if (graph->matrix[i] != NULL)
			free(graph->matrix[i]);
	}
	if (graph->matrix != NULL)
		free(graph->matrix);
	if (graph->visited != NULL)
		free(graph->visited);

	if (graph->values != NULL)
		free(graph->values);

	free(graph);
}

// Set an edge between vertices v1 and v2
void mgAddEdge(MatrixGraph *graph, int v1, int v2, int weight)
{
#ifdef _DEBUG
	if (weight == INVALID_WEIGHT)
		WARNING("Invalid weight inserted!!!");
#endif

	graph->matrix[v1][v2] = weight;
	if (graph->type == UNDIRECTED)
		graph->matrix[v2][v1] = weight;
}

// remove the edge between vertices v1 and v2
void mgRemoveEdge(MatrixGraph *graph, int v1, int v2)
{
	graph->matrix[v1][v2] = INVALID_WEIGHT;
	if (graph->type == UNDIRECTED)
		graph->matrix[v2][v1] = INVALID_WEIGHT;
}

// print the adjacency matrix
void mgPrint(MatrixGraph *graph)
{
	int i = 0;
	int j = 0;

	fprintf(stdout, "The Adjacency Matrix:\n");
	fprintf(stdout, "---------------------\n");
	

	/* Print the header */
	fprintf(stdout, "\t %11s", "node");
	
	for (i = 0; i < graph->numOfVertices; i++)
		fprintf(stdout, " %11c", graph->values[i]);
	fprintf(stdout, "\n");
	
	fprintf(stdout, "\t ___________");
	for (i = 0; i < graph->numOfVertices; i++)
		fprintf(stdout, "____________", graph->values[i]);
	fprintf(stdout, "\n");

	/* Print the values of the cells*/
	for (i = 0; i < graph->numOfVertices; i++) {
		fprintf(stdout, "\t");
		fprintf(stdout, "%11c|", graph->values[i]);
		for (j = 0; j < graph->numOfVertices; j++) {
			fprintf(stdout, " %11d", graph->matrix[i][j]);
		}
		fprintf(stdout, "\n");
	}

	/* Print the edges of the graph */
	fprintf(stdout, "The Edges:\n");
	fprintf(stdout, "----------\n");
	
	for (i = 0; i < graph->numOfVertices; i++) {
		fprintf(stdout, "\t");
		for (j = 0; j < graph->numOfVertices; j++) {
			if (graph->matrix[i][j] != INVALID_WEIGHT)
				fprintf(stdout, "%c-%c ", graph->values[i], graph->values[j]);
		}
		fprintf(stdout, "\n");
	}
}

// Visit all the nodes of the graph starting from v, in Depth First Order
void mgDepthFirstSearch(MatrixGraph *graph, int v)
{
	/////////////////////////////////////////
	// INSERT CODE HERE
	/////////////////////////////////////////
}

// Recursive DFS algorithm
void mgDFS(MatrixGraph *graph, int v)
{
	/////////////////////////////////////////
	// INSERT CODE HERE
	/////////////////////////////////////////
}

// Visit all the nodes of the graph starting from v, in Breadth First Order
void mgBreadthFirstSearch(MatrixGraph *graph, int v)
{
	/////////////////////////////////////////
	// INSERT CODE HERE
	/////////////////////////////////////////
}