#ifndef __MATRIX_GRAPH_H__
#define __MATRIX_GRAPH_H__

#include "defns.h"


/**
 * Represent a Graph as an Adjacent Matrix.
 */
typedef struct _MatrixGraph {
	GRAPHTYPE type;			// Unidirected or directed?
	int **matrix;			// The adjacency matrix
	int numOfVertices;		// The number of graph vertices (nodes)
	BOOL *visited;			// Indicated if  node was visited during a search or not
	char *values;			// The values for each node
} MatrixGraph;

// create and initialize a new graph
MatrixGraph *mgCreate(GRAPHTYPE type, int numOfVertices);

// Delete the graph
void mgDelete(MatrixGraph *graph);

// Set an edge with specified weight between vertices v1 and v2
// If the graph is directed, then the edge is from v1 to v2
// If the graph is undirected, the edge is unidirectional
void mgAddEdge(MatrixGraph *graph, int v1, int v2, int weight);

// Delete the edge between vertices v1 and v2
// If the graph is directed, then the edge is from v1 to v2
// If the graph is undirected, the edge is unidirectional
void mgRemoveEdge(MatrixGraph *graph, int v1, int v2);

// print the adjacency matrix
void mgPrint(MatrixGraph *graph);

// Visit all the nodes of the graph starting from v, in Depth First Order
void mgDepthFirstSearch(MatrixGraph *graph, int v);
void mgDFS(MatrixGraph *graph, int v);

// Visit all the nodes of the graph starting from v, in Breadth First Order
void mgBreadthFirstSearch(MatrixGraph *graph, int v);

#endif // __MATRIX_GRAPH_H__